@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
    Copyright (C) 2004 Arend van Beelen jr. <arend@auton.nl>
+   Copyright (C) 2009 Fredy Yanardi <fyanardi@gmail.com>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
@@ -19,10 +20,13 @@
 
 #include "searchbar.h"
 
+#include "SearchSuggestion.h"
+
 #include <unistd.h>
 #include <QLineEdit>
 #include <QApplication>
 #include <kaction.h>
+#include <KCompletionBox>
 #include <kconfig.h>
 #include <ksharedconfig.h>
 #include <kdebug.h>
@@ -46,6 +50,8 @@
 #include <qstyle.h>
 #include <QPixmap>
 #include <QMouseEvent>
+#include <QtGui/QCompleter>
+#include <QtCore/QTimer>
 
 K_PLUGIN_FACTORY(SearchBarPluginFactory, registerPlugin<SearchBarPlugin>();)
 K_EXPORT_PLUGIN(SearchBarPluginFactory("searchbarplugin"))
@@ -56,7 +62,8 @@ SearchBarPlugin::SearchBarPlugin(QObject *parent,
     m_searchCombo(0),
     m_searchMode(UseSearchProvider),
     m_urlEnterLock(false),
-    m_process(0)
+    m_process(0),
+    m_searchSuggestion(new SearchSuggestion(this))
 {
     m_searchCombo = new SearchBarCombo(0);
     m_searchCombo->lineEdit()->installEventFilter(this);
@@ -65,6 +72,7 @@ SearchBarPlugin::SearchBarPlugin(QObject *parent,
     connect(m_searchCombo, SIGNAL(iconClicked()), SLOT(showSelectionMenu()));
     m_searchCombo->setWhatsThis(i18n("Search Bar<p>"
                                      "Enter a search term. Click on the icon to change search mode or provider.</p>"));
+    connect(m_searchCombo, SIGNAL(suggestionEnabled(bool)), this, SLOT(enableSuggestion(bool)));
 
     m_popupMenu = 0;
 
@@ -73,7 +81,6 @@ SearchBarPlugin::SearchBarPlugin(QObject *parent,
     m_searchComboAction->setDefaultWidget(m_searchCombo);
     m_searchComboAction->setShortcutConfigurable(false);
 
-
     KAction *a = actionCollection()->addAction("focus_search_bar");
     a->setText(i18n("Focus Searchbar"));
     a->setShortcut(Qt::CTRL+Qt::Key_S);
@@ -83,6 +90,12 @@ SearchBarPlugin::SearchBarPlugin(QObject *parent,
 
     // parent is the KonqMainWindow and we want to listen to PartActivateEvent events.
     parent->installEventFilter(this);
+    connect(m_searchCombo->lineEdit(), SIGNAL(textEdited(const QString &)), SLOT(searchTextChanged(const QString &)));
+    connect(m_searchSuggestion, SIGNAL(suggestionReceived(const QStringList &)),
+            SLOT(addSearchSuggestion(const QStringList &)));
+    m_timer = new QTimer(this);
+    m_timer->setSingleShot(true);
+    connect(m_timer, SIGNAL(timeout()), SLOT(requestSuggestion()));
 }
 
 SearchBarPlugin::~SearchBarPlugin()
@@ -90,6 +103,7 @@ SearchBarPlugin::~SearchBarPlugin()
     KConfigGroup config(KGlobal::config(), "SearchBar");
     config.writeEntry("Mode", (int) m_searchMode);
     config.writeEntry("CurrentEngine", m_currentEngine);
+    config.writeEntry("SuggestionEnabled", m_suggestionEnabled);
 
     delete m_searchCombo;
     m_searchCombo = 0L;
@@ -177,8 +191,8 @@ void SearchBarPlugin::startSearch(const QString &search)
         KUriFilterData data;
         QStringList list;
         list << "kurisearchfilter" << "kuriikwsfilter";
-
-        service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
+        if ( !m_currentEngine.isEmpty() )
+            service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
         if (service) {
             const QString searchProviderPrefix = service->property("Keys").toStringList().first() + m_delimiter;
             data.setData(searchProviderPrefix + search);
@@ -219,8 +233,8 @@ void SearchBarPlugin::setIcon()
         KUriFilterData data;
         QStringList list;
         list << "kurisearchfilter" << "kuriikwsfilter";
-
-        service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
+        if ( !m_currentEngine.isEmpty() )
+            service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(m_currentEngine));
         if (service) {
             const QString searchProviderPrefix = service->property("Keys").toStringList().first() + m_delimiter;
             data.setData(searchProviderPrefix + "some keyword");
@@ -281,7 +295,7 @@ void SearchBarPlugin::showSelectionMenu()
             if (KUriFilter::self()->filterUri(data, list))
             {
                 QIcon icon;
-                QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
+                const QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
                 if (iconPath.isEmpty()) {
                     icon = KIcon("unknown");
                 } else {
@@ -331,7 +345,7 @@ void SearchBarPlugin::selectSearchEngines()
     m_process->start();
     if(!m_process->waitForStarted())
     {
-        kDebug(1202) << "Couldn't invoke kcmshell";
+        kDebug(1202) << "Couldn't invoke kcmshell4";
         delete m_process;
         m_process = 0;
     }
@@ -346,7 +360,7 @@ void SearchBarPlugin::searchEnginesSelected(int exitCode, QProcess::ExitStatus e
         config.sync();
         configurationChanged();
     }
-    delete m_process;
+    m_process->deleteLater();
     m_process = 0;
 }
 
@@ -382,6 +396,8 @@ void SearchBarPlugin::configurationChanged()
     if (m_currentEngine.isEmpty())
         m_currentEngine = "google";
 
+    m_suggestionEnabled = config.readEntry("SuggestionEnabled", true);
+    m_searchCombo->setSuggestionEnabled(m_suggestionEnabled);
     setIcon();
 }
 
@@ -409,8 +425,39 @@ void SearchBarPlugin::focusSearchbar()
     m_searchCombo->setFocus(Qt::ShortcutFocusReason);
 }
 
+void SearchBarPlugin::searchTextChanged(const QString &text)
+{
+    m_searchCombo->clearSuggestions();
+
+    if (m_suggestionEnabled && m_searchMode != FindInThisPage &&
+            m_searchSuggestion->isEngineAvailable(m_currentEngine) && !text.isEmpty()) {
+        if (m_timer->isActive()) {
+            m_timer->stop();
+        }
+
+        // 400 ms delay before requesting for suggestions, so we don't flood the provider with suggestion request
+        m_timer->start(400);
+    }
+}
+
+void SearchBarPlugin::requestSuggestion() {
+    if (!m_searchCombo->lineEdit()->text().isEmpty()) {
+        m_searchSuggestion->requestSuggestion(m_currentEngine, m_searchCombo->lineEdit()->text());
+    }
+}
+
+void SearchBarPlugin::enableSuggestion(bool enable)
+{
+    m_suggestionEnabled = enable;
+}
+
+void SearchBarPlugin::addSearchSuggestion(const QStringList &suggestions)
+{
+    m_searchCombo->setSuggestionItems(suggestions);
+}
+
 SearchBarCombo::SearchBarCombo(QWidget *parent) :
-    KHistoryComboBox(parent),
+    KHistoryComboBox(true, parent),
     m_pluginActive(true)
 {
     setDuplicatesEnabled(false);
@@ -420,10 +467,18 @@ SearchBarCombo::SearchBarCombo(QWidget *parent) :
     Q_ASSERT(useCompletion());
 
     KConfigGroup config(KGlobal::config(), "SearchBar");
-    QStringList list = config.readEntry( "History list", QStringList() );
-    list.prepend(QString()); // empty item
+    const QStringList list = config.readEntry( "History list", QStringList() );
     setHistoryItems(list, true);
     Q_ASSERT(currentText().isEmpty()); // KHistoryComboBox calls clearEditText
+
+    m_enableAction = new QAction(i18n("Enable Suggestion"), this);
+    m_enableAction->setCheckable(true);
+    connect(m_enableAction, SIGNAL(toggled(bool)), this, SIGNAL(suggestionEnabled(bool)));
+
+    connect(this, SIGNAL(aboutToShowContextMenu(QMenu*)), SLOT(addEnableMenuItem(QMenu*)));
+
+    // use our own item delegate to display our fancy stuff :D
+    completionBox()->setItemDelegate(new SearchBarItemDelegate(this)); 
 }
 
 const QPixmap &SearchBarCombo::icon() const
@@ -445,6 +500,11 @@ void SearchBarCombo::setIcon(const QPixmap &icon)
     setEditText(editText);
 }
 
+void SearchBarCombo::setSuggestionEnabled(bool enable)
+{
+    m_enableAction->setChecked(enable);
+}
+
 int SearchBarCombo::findHistoryItem(const QString &searchText)
 {
     for(int i = 0; i < count(); i++) {
@@ -480,6 +540,37 @@ void SearchBarCombo::setPluginActive(bool pluginActive)
     m_pluginActive = pluginActive;
 }
 
+void SearchBarCombo::setSuggestionItems(const QStringList &suggestions)
+{
+    if (!m_suggestions.isEmpty()) {
+        clearSuggestions();
+    }
+
+    m_suggestions = suggestions;
+    if (!suggestions.isEmpty()) {
+        const int size = completionBox()->count();
+        QListWidgetItem *item = new QListWidgetItem(suggestions.at(0));
+        item->setData(Qt::UserRole, "suggestion");
+        completionBox()->insertItem(size + 1, item);
+	const int suggestionCount = suggestions.count();
+        for (int i = 1; i < suggestionCount; i++) {
+            completionBox()->insertItem(size + 1 + i, suggestions.at(i));
+        }
+        completionBox()->popup();
+    }
+}
+
+void SearchBarCombo::clearSuggestions()
+{
+    int size = completionBox()->count();
+    if (!m_suggestions.isEmpty() && size >= m_suggestions.count()) {
+        for (int i = size - 1; i >= size - m_suggestions.size(); i--) {
+            completionBox()->takeItem(i);
+        }
+    }
+    m_suggestions.clear();
+}
+
 void SearchBarCombo::show()
 {
     if (m_pluginActive) {
@@ -487,10 +578,62 @@ void SearchBarCombo::show()
     }
 }
 
+void SearchBarCombo::addEnableMenuItem(QMenu *menu)
+{
+    if (menu) {
+        menu->addAction(m_enableAction);
+    }
+}
+
 SearchBarCombo::~SearchBarCombo()
 {
     KConfigGroup config(KGlobal::config(), "SearchBar");
     config.writeEntry( "History list", historyItems() );
+
+    delete m_enableAction;
+}
+
+SearchBarItemDelegate::SearchBarItemDelegate(QObject *parent)
+    : QItemDelegate(parent)
+{
+}
+
+void SearchBarItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+{
+    QString userText = index.data(Qt::UserRole).toString();
+    QString text = index.data(Qt::DisplayRole).toString();
+
+    // Get item data
+    if (!userText.isEmpty()) {
+        // This font is for the "information" text, small size + italic + gray in color
+        QFont usrTxtFont = option.font;
+        usrTxtFont.setItalic(true);
+        usrTxtFont.setPointSize(6);
+
+        QFontMetrics usrTxtFontMetrics(usrTxtFont);
+        int width = usrTxtFontMetrics.width(userText);
+        QRect rect(option.rect.x(), option.rect.y(), option.rect.width() - width, option.rect.height());
+        QFontMetrics textFontMetrics(option.font);
+        QString elidedText = textFontMetrics.elidedText(text,
+                Qt::ElideRight, option.rect.width() - width - option.decorationSize.width());
+
+        QAbstractItemModel *itemModel = const_cast<QAbstractItemModel *>(index.model());
+        itemModel->setData(index, elidedText, Qt::DisplayRole);
+        QItemDelegate::paint(painter, option, index);
+        itemModel->setData(index, text, Qt::DisplayRole);
+
+        painter->setFont(usrTxtFont);
+        painter->setPen(QPen(QColor(Qt::gray)));
+        painter->drawText(option.rect, Qt::AlignRight, userText);
+
+        // Draw a separator above this item
+        if (index.row() > 0) {
+            painter->drawLine(option.rect.x(), option.rect.y(), option.rect.x() + option.rect.width(), option.rect.y());
+        }
+    }
+    else {
+        QItemDelegate::paint(painter, option, index);
+    }
 }
 
 #include "searchbar.moc"
